# costfunction.m - Calculate Cost Function
# Author:   Richard James MacPherson and Grant Ingram
# Revised Michael Hilfer

# Copyright 2009-2012  Richard James MacPherson, Grant Ingram, 
# Jamie McIntosh and Michael Hilfer.  
# This file is part of PEW V2.

# PEW is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PEW is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PEW.  If not, see <http://www.gnu.org/licenses/>.

#####################
# Revision History - Jamie McIntosh
# 1.01 - String inserted to be used in database functions
# 1.02 - Included extra variables to be considered in cost calculation
# 1.03 - Code improved to increase readability and to allow easy selection
#        of cost variables
# 1.04 - Variable weightings now can be set by user and are no longer hard
#        coded into functions
# 1.10 - Vega
#####################
# Known Issues
#
#####################

## Uncomment/Comment lines to modify cost function

function cost = costfunction(yaw,cske,ucubed,stream,massflow)

global cost_str;

## Flow Turning 
turning = (-1*yaw)+42.75;  % Determines the total angle the flow has been turned

if (111.845886-turning) > 0    % Calcuates yaw component
  f_yaw=(111.845886-turning);  % If the flow is underturned, penalise
else
  f_yaw=0;                     % If the flow is overturned, ignores
endif
% f_yaw = 0;                   % Comment this line to use flow turning in cost calculation

## Cske Variable
f_cske = (cske*100);                 % Calcuates Cske component
% f_cske = 0;                          % Comment this line to use cske in cost calculation

## U^3 Variable
#f_ucubed = (ucubed/1000);            % Calculates U^3 component
f_ucubed = 0;                      % Comment this line to use U^3 integral in cost calculation

## Streamline Data
#f_stream = stream;                    % Calculate dynamic head loss component 
f_stream = 0;                    % Comment this line to use streamline data in cost calculation

## Database Variable
cost_str = 'f_yaw+f_cske';              % String function used in export.m
% cost_str = 'f_yaw+f_ucubed';          
% cost_str = 'f_yaw+f_ucubed+f_stream'; 

## Massflowrate
massfplan = 1.1663
massflowdelta = abs(massfplan - massflow);
massflowproc = (massflowdelta/massfplan);
massflowprod = massflowproc * 100;
f_massf = massflowprod;
%f_massf = 0;

## Weightings
w_yaw = 1;
w_cske = 0.7;
w_ucubed = 1;
w_stream = 0.7;
w_massf = 0.7;

## Objective Function
cost = (w_yaw*f_yaw)+(w_cske*f_cske)+(w_ucubed*f_ucubed)+(w_stream*f_stream)+(w_massf*f_massf);        % Determines objective function
  if( isnan(cost))
  cost =1000;
  endif

## write cost parameter into a file
#filecost = fopen('costpar.dat','w');
#fprint(filecost, "cost parameter\n");
#fprint(filecost, "f_yaw=%9.5f f_cske=%9.5f f_ucubed=%9.5f f_stream=%9.5f f_massf=%9.5f\n",f_yaw,f_cske,f_ucubed,f_stream,f_massf);
#fprint(filecost, "cost=%9.5f\n",cost);
#fclose(filecost);

endfunction
