#! /usr/bin/octave 
## curve.m - Generates Fourier Series curves and Submits files to Hamilton
## Author: Richard James MacPherson and Grant Ingram
## Description: Generates Fourier Series Curves
##              Generates Hamilton Input Script
##              Generates Hamilton Submission Shell Script
##              Moves ICEM & FLUENT files into local folders
##              Submits files to Hamilton
## Notes: Before running check ALL file destinations
##        Also ssh keys need to be set up to allow pasword free login 

# Copyright 2009-2012  Richard James MacPherson, Grant Ingram, 
# Jamie McIntosh and Michael Hilfer.  
# This file is part of PEW V2.

# PEW is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PEW is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PEW.  If not, see <http://www.gnu.org/licenses/>.

#####################
# Revision History - Jamie McIntosh
# 1.01 - Fourier curve generation fixed for yval, now uses correct
#        parameters in each chromosome
# 1.02 - Scaling fixed and first generation created as intended
# 1.03 - ssh connection now possible from outside of university network
# 1.04 - Scaling method changed and now final. Working as intended for all
#        lengths of fourier series
# 1.05 - Survivors of each generation are copied into the next Generation
#        filestore so that repeat testing is removed and script modified to
#        skip these chromosomes in CFD analysis
# 1.06 - Reverted to 1.04 - skipping affects mutation
# 1.07 - Submit scripts modified to allow batching multiple jobs when cluster
#        load is high
# 1.08 - Curve modified to use devmyri.q instead of seq.q on cluster
# 1.09 - Integrated database choice option and space saver
# 1.10 - Vega script added to process fvuns file using fieldview
# 1.11 - Non intensive jobs, ie server up/downloading now performed on vega
# 1.12 - Vega + Hamilton
#
#####################
# Known Issues
#
#####################

function curve() % generates fourier series curves and input files 

global iga; global harmonics; global popsize; global npar;
global heighthi; global heightlow; global par; global pitch;
global base_dir; global cluster_dir; global user_id;
global job_id; global keep; global nbatch; global datalink;

## Location of Axial Planes and Curve Starting Vertex Locations
axial=[-204;-164.32;-126.4;-86.67;-48.75;-9.03];
startpoint=[217.82; 253.91; 270.29; 257.46; 208.6; 114.87];

##  Generates the Generation File in the Current Working Directory 
newdir = sprintf("Generation_%d", iga); 
mkdir(newdir); cd(newdir);
disp(sprintf("Generation %d",iga));

## Generates the Chromosome File & Curve Files 

for chrom=1:1:popsize                % Cycle through chromosomes
  newdir = sprintf("chr_%d", chrom); % Generates the chromosome file
  mkdir(newdir);
  cd(newdir);

  for line=0:1:5          % Cycle through axial planes
    xval = 0:0.01:(2*pi); % Initialise xvalue in range (0 - 2pi) at 0.01 radians
    yval = 0; ymax = 0;   % Initialise y and ymax values
 
    for harm=1:1:(harmonics) % Generates Fourier series curve
            yval = yval + ((par(chrom,(harm+(((harmonics*2)+1)*line))))*sin(harm*xval)) + ((par(chrom,((harm+harmonics)+(((harmonics*2)+1)*line))))*cos(harm*xval));
    end
    
    ymax = max(abs(max(yval)), abs(min(yval))); % Finds maximum value for scaling

    ## Scales the Fourier series in relation to maximum pertubation height  
    yval = yval*(par(chrom,((npar/6)+((npar/6)*line)))/ymax); % Scales line between -1 & 1 and 
							      % then scaled by the par amplitude

    ## Scales the curves to fit within domain
    xval = xval*pitch/(2*pi);
    xval = xval + (startpoint(line+1)-pitch);

    ## Generates files to store Fourier Series Curves

    filename = ['line_' int2str(line+1) '.csv'];
    [fid] = fopen(filename, 'w');
       for o=1:1:629
         fprintf(fid,'%3.6f', axial(line+1));
         fprintf(fid,'%s',',');
         fprintf(fid,'%3.6f', xval(o));
         fprintf(fid,'%s',',');
         fprintf(fid,'%2.6f\n', yval(o));
       end
    fclose(fid);
  end
##Michael: Copies Pressure and Suction Blade Side Files from current working direction to chr directory
  sypr = sprintf("cp %spressureside.txt .",base_dir);
  sysu = sprintf("cp %ssuctionside.txt .",base_dir);
#sy1=sprintf("cp %sGeneration_%d/chr_%d/chrom.ip %sip/%d.ip",base_dir,iga,chrom,data_root,data_size+1+count);
  system(sypr);
  system(sysu);
  ## Copies ICEM script and pre generated blocking strategy for Icem to chr directory
  sysicem1 =  sprintf("cp %sicemreplay.rpl .",base_dir);
  sysicem2 =  sprintf("cp %sfinal.blk .",base_dir);
  system(sysicem1);
  system(sysicem2);
  ## Copies Initialdata
 # sysicem1 =  sprintf("cp %sinitial.cas .",base_dir);
 # sysicem2 =  sprintf("cp %sinitial.ip .",base_dir);
 # system(sysicem1);
 # system(sysicem2);
  ## Generate Endwall Script for ICEM
######################################################################
## Author:  Michael Hilfer                             	            ##
## This script part reads curved lines and generates space limited  ##
## control lines, blade pressure and suction side profiles          ##
######################################################################
################### General Parameter  ###############################

numoflines.curved=6 %number of curved lines
numoflines.controlu=6 %number of upstream control lines
numoflines.controld=6 %number of downstream control lines
numoflines.all=20 % 6 + 6 + 6 + inlet line + outlet line
leadedge.x=-177.97518298
leadedge.y=138.57831553
trailedge.x=-1.27825746
trailedge.y=-1.8787054
curlinemid.a=122.32
curlinemid.g=19.37
dist.edge=60
angle.inlet=0
angle.outlet=0
dist.upstream=1        % in axial chord length
dist.downstream=2       % in axial chord length
pitch=191
span=375

########## Reading Curved Lines and Profile Lines ###############

curvedline.a.a = dlmread ('line_1.csv',',')
curvedline.b.a = dlmread ('line_2.csv',',')
curvedline.c.a = dlmread ('line_3.csv',',')
curvedline.d.a = dlmread ('line_4.csv',',')
curvedline.e.a = dlmread ('line_5.csv',',')
curvedline.f.a = dlmread ('line_6.csv',',')
suction.a = dlmread ('suctionside.txt',',')
pressure.a = dlmread ('pressureside.txt',',')

####################### Dimension of Lines ######################

curvedline.a.rows = rows (curvedline.a.a)
curvedline.b.rows = rows (curvedline.b.a)
curvedline.c.rows = rows (curvedline.c.a)
curvedline.d.rows = rows (curvedline.d.a)
curvedline.e.rows = rows (curvedline.e.a)
curvedline.f.rows = rows (curvedline.f.a)
suction.rows = rows (suction.a)
pressure.rows = rows (pressure.a)

##################### Axial Chord Length  #######################

range.axial.ma = max (suction.a(:,2))
range.axial.mi = min (suction.a(:,2))
range.axial.delta = abs(abs(range.axial.ma) - abs(range.axial.mi))

#################### Inlet and Outlet Lines #####################
# The Geometry is modified to support current meshing           #
# strategy                                                      #
#################################################################

inout1 = zeros (curvedline.a.rows , 3)
inout2 = zeros (curvedline.f.rows , 3)

inlet.startpoint.x = leadedge.x - (dist.upstream * range.axial.delta) 
inlet.startpoint.y = leadedge.y - (leadedge.y - curlinemid.a + 10)
	inout1(1,1)=inlet.startpoint.x
	inout1(1,2)=inlet.startpoint.y - (pitch / 2)
	inout1(curvedline.a.rows,1)=inlet.startpoint.x
	inout1(curvedline.a.rows,2)=inlet.startpoint.y + (pitch / 2)
	
	dy=pitch/curvedline.a.rows;
	loop1=curvedline.a.rows-1;	
	
           for i = 2:1:loop1;
           inout1(i, 1)=inlet.startpoint.x;
	   end;
              for i = 2:1:loop1;
		h=i-1;
              inout1(i ,2) = inout1(h, 2) + dy;
	      end;

outlet.startpoint.x = trailedge.x + (dist.downstream * range.axial.delta) 
outlet.startpoint.y = trailedge.y + (trailedge.y - curlinemid.g - 10)
	inout2(1,1)=outlet.startpoint.x
	inout2(1,2)=outlet.startpoint.y - (pitch / 2)
	inout2(curvedline.f.rows,1)=outlet.startpoint.x
	inout2(curvedline.f.rows,2)=outlet.startpoint.y + (pitch / 2)
	
	dy=pitch/curvedline.f.rows;
	loop1=curvedline.f.rows-1;	
	
           for i = 2:1:loop1;
           inout2(i, 1)=outlet.startpoint.x;
	   end;
              for i = 2:1:loop1;
		h=i-1;
              inout2(i ,2) = inout2(h, 2) + dy;
	      end;

#################################################################
#### Writing Space Limited Files for GridGen/Pointwise/ICEM #####
#### The generated Files are readable by Pointwise/GG/ICEM  #####
#################################################################

######################### Inlet File ############################	
file1 = fopen('newicemin.sldcrv','w');
fprintf(file1, "%i %i\n", curvedline.a.rows, numoflines.all);
	for i = 1:curvedline.a.rows;
           fprintf(file1, "%9.5f %9.5f %9.5f\n", inout1(i, 1), inout1(i, 2), inout1(i, 3));
	end;
#################### Inlet Control Lines #######################
controlline = zeros (curvedline.a.rows , 3);
ii=numoflines.controlu;
for i = 1:1:ii;
	deltainleadx = (inlet.startpoint.x + 241)  / numoflines.controlu;
	deltainleadx1 = deltainleadx * i;
	conrtol.startpoint.x = inlet.startpoint.x - deltainleadx1;
	conrtol.startpoint.x

	deltainleady = 0;
	deltainleady1 = deltainleady * i
	conrtol.startpoint.y = inout1(1,2) + deltainleady1;
	conrtol.startpoint.y

	controlline(1,1) = conrtol.startpoint.x
	controlline(1,2) = conrtol.startpoint.y
	controlline(curvedline.a.rows,1) = conrtol.startpoint.x
	controlline(curvedline.a.rows,2) = controlline(1,2) + 191

	dy = pitch/curvedline.a.rows;
	loop1 = curvedline.a.rows - 1;	

          for j = 2:1:loop1;
         	controlline(j, 1)=conrtol.startpoint.x;
	   end;
          for j = 2:1:loop1;
		h=j-1;
           	controlline(j ,2) = controlline(h, 2) + dy;
	   end;

	for j = 1:curvedline.a.rows;
          fprintf(file1, "%9.5f %9.5f %9.5f\n", 
          controlline(j, 1), controlline(j, 2), controlline(j, 3));
	end;
end;	
######################### Line 1-6 ##############################
	for i = 1:curvedline.a.rows;
           fprintf(file1, "%9.5f %9.5f %9.5f\n", curvedline.a.a(i, 1), curvedline.a.a(i, 2), curvedline.a.a(i, 3));
	end;
	for i = 1:curvedline.a.rows;
           fprintf(file1, "%9.5f %9.5f %9.5f\n", curvedline.b.a(i, 1), curvedline.b.a(i, 2), curvedline.b.a(i, 3));
	end;
	for i = 1:curvedline.a.rows;
           fprintf(file1, "%9.5f %9.5f %9.5f\n", curvedline.c.a(i, 1), curvedline.c.a(i, 2), curvedline.c.a(i, 3));
	end;		
	for i = 1:curvedline.a.rows;
           fprintf(file1, "%9.5f %9.5f %9.5f\n", curvedline.d.a(i, 1), curvedline.d.a(i, 2), curvedline.d.a(i, 3));
	end;		
	for i = 1:curvedline.a.rows;
           fprintf(file1, "%9.5f %9.5f %9.5f\n", curvedline.e.a(i, 1), curvedline.e.a(i, 2), curvedline.e.a(i, 3));
	end;		
	for i = 1:curvedline.a.rows;
           fprintf(file1, "%9.5f %9.5f %9.5f\n", curvedline.f.a(i, 1), curvedline.f.a(i, 2), curvedline.f.a(i, 3));
	end;		
##################### Outlet Control Lines ######################

for i = 1:numoflines.controld;

	cc = numoflines.controld - i + 1;
	deltaintrailx = (outlet.startpoint.x - 60) / numoflines.controld;
	deltaintrailx1 = deltaintrailx * cc;
	conrtol.startpoint.x = outlet.startpoint.x - deltaintrailx1;

	deltaintraily = 0;
	deltaintraily1 = deltaintraily * cc
	conrtol.startpoint.y = inout2(1,2) + deltaintraily1;

	controlline(1,1) = conrtol.startpoint.x
	controlline(1,2) = conrtol.startpoint.y
	controlline(curvedline.f.rows,1) = conrtol.startpoint.x
	controlline(curvedline.f.rows,2) = controlline(1,2) + 191
	
	dy = pitch/curvedline.f.rows;
	loop1 = curvedline.f.rows - 1;	
	
           for j = 2:1:loop1;
           controlline(j, 1)=conrtol.startpoint.x;
	   end;
              for j = 2:1:loop1;
		h=j-1;
              controlline(j ,2) = controlline(h, 2) + dy;
	      end;
	for j = 1:curvedline.f.rows;
          fprintf(file1, "%9.5f %9.5f %9.5f\n", 
          controlline(j, 1), controlline(j, 2), controlline(j, 3));
	end;

end;
########################### Outlet Line #########################

	for i = 1:curvedline.a.rows;
           ii=i+2000;
           fprintf(file1, "%9.5f %9.5f %9.5f\n", inout2(i, 1), inout2(i, 2), inout2(i, 3));
	end;		
fclose(file1);
########################## suction ##############################
% suction.dat = blade suction side points
file9 = fopen('suction.dat','w');
	fprintf(file9, "%i\n",suction.rows);
	for i = 1:suction.rows;
           fprintf(file9, "%9.5f %9.5f %9.5f\n", suction.a(i, 1), suction.a(i, 2), -40);
	end;	
fclose(file9);
########################## pressure #############################
% pressure.dat = blade pressure side points
file10 = fopen('pressure.dat','w');
	fprintf(file10, "%i\n",pressure.rows);
	for i = 1:pressure.rows;
           fprintf(file10, "%9.5f %9.5f %9.5f\n", pressure.a(i, 1), pressure.a(i, 2), -40);
	end;
fclose(file10);

#################################################################
## End Generating End-Wall Curves                              ##
#################################################################

  disp(sprintf("Created Endwall Curve Files for chr %d",chrom));
  disp(sprintf("Created Endwall Curve File  for ICEM and chr %d",chrom));
  disp(sprintf(pwd));
  ## Copies ICEM & FLUENT Files from current working directory to chr directory
 
  if datalink == 0 % dont use database - space saving
    sy2 = sprintf("cp %sinputfile ./inputfile",base_dir); % Planar Case, space saving mode
  else
    sy2 = sprintf("cp %sinputfile ./inputfile",base_dir); % Most Similar Case
  endif
  system(sy2);

  sy3 = sprintf("cp %sfv* .",base_dir); % Fieldview Streamline Post-Processing Files
  system(sy3);

  ## Generate Script that runs Post-Processing on Vega
  filename = ['fv_process.csh'];
  [fid] = fopen(filename, 'w');
  fprintf(fid,"#!/bin/csh \n");
  fprintf(fid,"# %s script generated by curve.m \n",filename);
  fprintf(fid,"ssh -Y %s@vega3 ",user_id); 
  fprintf(fid,"'"); % open the command list in the script
  fprintf(fid,"cd %sGeneration_%d/chr_%d/ ;",base_dir,iga,chrom)
  fprintf(fid,"fv -batch -fvx fvprocess.fvx >& fv.out");
  fprintf(fid,"'"); % close the command list in the script
  fclose(fid);

  cd('..'); % end of chromosomes
end

## Process Population on Hamilton 
batchid = 0;
chrom_count = 0;
while(chrom_count<popsize)
  batchid++; % Current batch number
  ## Generate Hamilton INPUT File
  filename = ['Gen_' int2str(iga) 'chrs_' int2str(chrom_count+1) '-' int2str(chrom_count+nbatch) '.csh'];
  [fid] = fopen(filename, 'w');
  fprintf(fid,"#!/bin/csh \n");
  fprintf(fid,"# %s script generated by curve.m \n",filename);
  fprintf(fid,"# use current working directory \n");
  fprintf(fid,"#$ -cwd \n\n");
  fprintf(fid,"# combine standard error and output files \n");
  fprintf(fid,"#$ -j y \n\n");
  fprintf(fid,"# request submission to a queue for serial jobs \n");
  fprintf(fid,"#$ -q long.q \n\n"); 
  fprintf(fid,"# execute commands \n");
  fprintf(fid,"module load fluent \n"); 
  i=0; % for loop counter
  for batch=1:1:nbatch
    chrom_count++;
    i++;
    if chrom_count > popsize
      break % Stops loop when population complete
    endif   
    fprintf(fid,"cd chr_%d \n",chrom_count);
    fprintf(fid,"/hpsfs/local/Cluster-Apps/fluent/ansys_inc/v121/icemcfd/linux64_amd/bin/icemcfd -batch -script icemreplay.rpl\n");    %call icem        
    fprintf(fid,"/hpsfs/local/Cluster-Apps/fluent/ansys_inc/v121/fluent/fluent12.1.4/bin/fluent 3d -g -sge -i inputfile\n");           %call fluent
	fprintf(fid,"touch fluenterror.log\n");
	fprintf(fid,"rm base.cas \n"); %space save mode
	fprintf(fid,"rm base.fvuns \n"); %space save mode
	fprintf(fid,"rm hex.uns \n"); %space save mode
	fprintf(fid,"rm project1.uns \n"); %space save mode
    fprintf(fid,"cd .. \n\n");
  endfor
  fclose(fid);
  chrom_count = chrom_count - i; % return counter to correct start value
  		
  ## Generate submit script for Hamilton
  marker = ['submit_' int2str(batchid)];
  [fid] = fopen(marker, 'w');
  fprintf(fid,"#!/bin/bash \n");
  for batch=1:1:nbatch
    chrom_count++;
    if chrom_count > popsize
      break % Stops loop when population complete
    endif  
    fprintf(fid,"ssh %s@hamilton1.dur.ac.uk 'mkdir -p %sGeneration_%d/chr_%d/'\n"
  	    ,user_id,cluster_dir,iga,chrom_count);
    fprintf(fid,"scp %sGeneration_%d/chr_%d/* %s@hamilton1.dur.ac.uk:%sGeneration_%d/chr_%d/\n"
	    ,base_dir,iga,chrom_count,user_id,cluster_dir,iga,chrom_count);
  endfor
  fprintf(fid,"ssh %s@hamilton1.dur.ac.uk ",user_id);
  fprintf(fid,"'"); % open the command list in the script
  fprintf(fid,"setenv SGE_ROOT /usr/local/Cluster-Apps/sge/default_20110330 ;"); %is needed on Hamilton.......
  fprintf(fid,"module load fluent ;"); 
  fprintf(fid,"cd %sGeneration_%d/ ;",cluster_dir,iga);
  fprintf(fid,"qsub -q long.q %sGeneration_%d/Gen_%dchrs_%d-%d.csh",cluster_dir,iga,iga,chrom_count-i+1,chrom_count-i+nbatch); %job submition
  fprintf(fid,"'"); % close the command list in the script
  fclose(fid);

endwhile

## Submit Jobs to Hamilton
sy4 = sprintf("ssh %s@hamilton1.dur.ac.uk 'mkdir -p %sGeneration_%d/'",user_id,cluster_dir,iga);
system(sy4); % create generation filestore fpr script files
sy5 = sprintf("scp -pq Gen_* %s@hamilton1.dur.ac.uk:%sGeneration_%d/",user_id,cluster_dir,iga);
system(sy5); % copy script files to hamilton 
for batch=1:1:batchid
  sy6 = sprintf("bash submit_%d",batch); %start job
  [status, result]=system(sy6);

  ## Logs Hamilton job_id number
  if status == 0
	  idx = regexp(result,'\d+');
	  nums = regexp(result,'\d+','match'); 
	  job=str2num(nums{1});
	  job_id(batch,1)=job;
	  disp(['Submitted job, job_id number:' int2str(job)]);
  endif
endfor

cd('..'); % end of generation
endfunction
