## monitor.m - Checks the status of the cluster 
## Author: Grant Ingram
## Revised: Michael Hilfer
## Notes: Before running check ALL file destinations
##        Also ssh keys need to be set up to allow pasword free login 
#####################

# Copyright 2009-2012  Richard James MacPherson, Grant Ingram, 
# Jamie McIntosh and Michael Hilfer.  
# This file is part of PEW V2.

# PEW is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PEW is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PEW.  If not, see <http://www.gnu.org/licenses/>.

# Revision History - Jamie McIntosh
# 1.01 - ssh connection now possible from outside of university network
# 1.02 - Survivors of each generation are copied into the next Generation
#        filestore so that repeat testing is removed and script modified to
#        skip these chromosomes in CFD analysis
# 1.03 - Reverted to 1.01 - skipping affects mutation
# 1.04 - Monitor adapted to scan batched jobs individually
# 1.06b - THIS VERSION HAS NO FV SUPPORT AND RUNS ON VEGA
# 1.1 - Redesigned to run on Vega
# 1.12  Reversed 1.1 to run on Hamilton
#####################
# Known Issues
#
#####################

function monitor() 

global iga; global base_dir; global cluster_dir; global job_id;
global popsize; global user_id; global keep;

printf("Monitoring the job status...");
tic();  # Start timer for the generation
donejobs = 0; 
while (1)
  printf("Hamilton Jobs:"); %changed to Vega and back to Hamilton
  printf(" %d",job_id(:,1)); % Display the jobs sent to Hamilton
  printf("\n");
  donejobs = 0; errornum = 0; # reset counters
  for chrom=1:1:(popsize) % Cycles through each chromosome
    printf("Chromosome %d:\t",chrom);
    chrom_dir = sprintf("%sGeneration_%d/chr_%d/",cluster_dir,iga,chrom);
    jobfile_icem = ['mesh_report.html']; %actually there is no .out job file for icem so a mesh_report file is generated automaticaly...
    jobfile_fluent = ['fluent.out']; 
    errorfile_fluent = ['fluenterror.log'];

    # Check Meshing Progress
    [status,output] = system(sprintf("ssh %s@hamilton1.dur.ac.uk 'test -e %s%s && echo 10'"
		     ,user_id,chrom_dir,jobfile_icem),1);

    if (output > 0 ) # The mesh is done
	printf("Mesh: Done\t");
    else 
      # No file so the system must still be waiting for a slot or running...
      printf("Mesh: Waiting or Running\t");
    endif

    # Check CFD Progress
    [status,output] = system(sprintf("ssh %s@hamilton1.dur.ac.uk 'test -e %s%s && echo 10'"
		     ,user_id,chrom_dir,jobfile_fluent),1);

    [status,output3] = system(sprintf("ssh %s@hamilton1.dur.ac.uk 'test -e %s%s && echo 10'"
				       ,user_id,chrom_dir,errorfile_fluent),1);

      if (output > 0 ) # The file exists or running, halted or done

      # Check for results file 
      [status,output1] = system(sprintf("ssh %s@hamilton1.dur.ac.uk 'test -e %s%s && echo 10'"
				       ,user_id,chrom_dir,"res"),1);
      if (output1 > 0)
	# Results files exist so job is done....
	disp(sprintf("CFD: Done"));
	donejobs = donejobs + 1;
     
      elseif (output3 > 0)
       
       disp(sprintf("CFD: Done with error"));
       donejobs = donejobs + 1;
       errornum = errornum +1;

       else
	# File exists, no results and not halted so we must still be running...
	disp(sprintf("CFD: Running"));
      endif
    elseif (output3 > 0)
      disp(sprintf("CFD: Done with error"));
	donejobs = donejobs + 1;
	errornum = errornum +1;
      else
      # No file so the system must still be waiting for a slot...
      disp(sprintf("CFD: Waiting"));
    endif

  endfor

  if (donejobs == popsize) 
    break
  else
    disp("*** Waiting 120 seconds ***");
    pause(120); 
  endif

endwhile
printf("All chromosomes done\n");
#disp('All chromosomes done');
elapsed_time = toc();
printf("\nGeneration %d took %f hours",iga,elapsed_time/3600);

endfunction
